@@ -15,18 +15,11 @@ sub new {
     my $class = shift;
     my $regfile = shift;
     my $offset = shift; # offset to RGDB value entry
-    my $parent_key_path = shift; # parent key path (for errors)
 
-    croak "Missing registry file" if !defined $regfile;
-    croak "Missing offset" if !defined $offset;
+    croak 'Missing registry file' if !defined $regfile;
+    croak 'Missing offset' if !defined $offset;
 
-    # when errors are encountered
-    my $whereabouts = defined($parent_key_path)
-                    ? " (a value of '$parent_key_path')"
-                    : "";
-
-    my $fh = $regfile->{_filehandle};
-    croak "Missing filehandle" if !defined $fh;
+    my $fh = $regfile->get_filehandle;
 
     # RGDB Value Entry
     # 0x00 dword = value type
@@ -38,29 +31,28 @@ sub new {
     # Value type may just be a word, not a dword;
     # following word always appears to be zero.
 
-	sysseek($fh, $offset, 0);
-    my $bytes_read = sysread($fh, my $rgdb_value_entry, RGDB_VALUE_HEADER_LENGTH);
+    sysseek($fh, $offset, 0);
+    my $bytes_read = sysread($fh, my $rgdb_value_entry,
+                             RGDB_VALUE_HEADER_LENGTH);
     if ($bytes_read != RGDB_VALUE_HEADER_LENGTH) {
-        warnf("Could not read RGDB value at 0x%x%s",
-            $offset, $whereabouts);
+        warnf('Could not read RGDB value at 0x%x', $offset);
         return;
     }
 
     my ($type,
         $name_length,
-        $data_length) = unpack("Vx4vv", $rgdb_value_entry);
+        $data_length) = unpack('Vx4vv', $rgdb_value_entry);
 
     $bytes_read = sysread($fh, my $name, $name_length);
     if ($bytes_read != $name_length) {
-        warnf("Could not read name for RGDB value at 0x%x%s",
-            $offset, $whereabouts);
+        warnf('Could not read name for RGDB value at 0x%x', $offset);
         return;
     }
+    $name = decode($Parse::Win32Registry::Base::CODEPAGE, $name);
 
     $bytes_read = sysread($fh, my $data, $data_length);
     if ($bytes_read != $data_length) {
-        warnf("Could not read data for RGDB value at 0x%x%s",
-            $offset, $whereabouts);
+        warnf('Could not read data for RGDB value at 0x%x', $offset);
         return;
     }
 
@@ -68,9 +60,10 @@ sub new {
     $self->{_regfile} = $regfile;
     $self->{_offset} = $offset;
     $self->{_length} = RGDB_VALUE_HEADER_LENGTH + $name_length + $data_length;
-    $self->{_allocated} = 0;
-    $self->{_tag} = 'rgdb';
+    $self->{_allocated} = 1;
+    $self->{_tag} = 'rgdb value';
     $self->{_name} = $name;
+    $self->{_name_length} = $name_length;
     $self->{_type} = $type;
     $self->{_data} = $data;
     $self->{_data_length} = $data_length;
@@ -83,15 +76,14 @@ sub get_data {
     my $self = shift;
 
     my $type = $self->get_type;
-    croak "Missing type" if !defined $type;
 
     my $data = $self->{_data};
-    return if !defined $data;
+    return if !defined $data; # actually, Win95 value data is always defined
 
     # apply decoding to appropriate data types
     if ($type == REG_DWORD) {
         if (length($data) == 4) {
-            $data = unpack("V", $data);
+            $data = unpack('V', $data);
         }
         else {
             # incorrect length for dword data
@@ -102,21 +94,15 @@ sub get_data {
         # Snip off any terminating null.
         # Typically, REG_SZ values will not have a terminating null,
         # while REG_EXPAND_SZ values will have a terminating null
-        my $last_char = substr($data, -1, 1);
-        if (ord($last_char) == 0) {
-            $data = substr($data, 0, length($data) - 1);
-        }
+        chop $data if substr($data, -1, 1) eq "\0";
     }
     elsif ($type == REG_MULTI_SZ) {
-        my @multi_sz = ();
-        my $pos = 0;
-        do {
-            my ($str, $str_len) = unpack_string(substr($data, $pos));
-            push @multi_sz, $str;
-            $pos += $str_len;
-        } while ($pos < length($data));
-        # drop trailing empty string (caused by trailing null)
-        pop @multi_sz if @multi_sz > 1 && $multi_sz[-1] eq '';
+        # Snip off any terminating nulls
+        chop $data if substr($data, -1, 1) eq "\0";
+        chop $data if substr($data, -1, 1) eq "\0";
+        my @multi_sz = split("\0", $data, -1);
+        # Make sure there is at least one empty string
+        @multi_sz = ('') if @multi_sz == 0;
         return wantarray ? @multi_sz : join($", @multi_sz);
     }
 
@@ -134,6 +120,7 @@ sub as_regedit_export {
 
     if ($type == REG_SZ) {
         $export .= '"' . $self->get_data . '"';
+        $export .= "\n";
     }
     elsif ($type == REG_BINARY) {
         $export .= 'hex:';
@@ -144,6 +131,7 @@ sub as_regedit_export {
         $export .= defined($data)
             ? sprintf("dword:%08x", $data)
             : "dword:";
+        $export .= "\n";
     }
     elsif ($type == REG_EXPAND_SZ || $type == REG_MULTI_SZ) {
         my $data = $version == 4
@@ -157,23 +145,18 @@ sub as_regedit_export {
         $export .= sprintf("hex(%x):", $type);
         $export .= format_octets($data, length($export));
     }
-    $export .= "\n";
     return $export;
 }
 
 sub parse_info {
     my $self = shift;
 
-    my $info = sprintf '0x%x,%d rgdb "%s" type=%s (%s) data=%d bytes',
+    my $info = sprintf '0x%x rgdb value len=0x%x "%s" type=%d data,len=0x%x',
         $self->{_offset},
         $self->{_length},
         $self->{_name},
         $self->{_type},
-        $self->get_type_as_string,
         $self->{_data_length};
-    if ($self->{_type} == REG_DWORD) {
-        $info .= sprintf ' "%s"', $self->get_data_as_string;
-    }
     return $info;
 }
 
