@@ -9,46 +9,35 @@ use Carp;
 use Encode;
 use Parse::Win32Registry::Base qw(:all);
 
-use constant OFFSET_TO_FIRST_HBIN => 0x1000;
 use constant VK_HEADER_LENGTH => 0x18;
+use constant OFFSET_TO_FIRST_HBIN => 0x1000;
 
 sub new {
     my $class = shift;
     my $regfile = shift;
     my $offset = shift; # offset to vk record relative to first hbin
-    my $parent_key_path = shift; # parent key path (for errors)
 
-    croak "Missing registry file" if !defined $regfile;
-    croak "Missing offset" if !defined $offset;
+    croak 'Missing registry file' if !defined $regfile;
+    croak 'Missing offset' if !defined $offset;
 
-    # when errors are encountered
-    my $whereabouts = defined($parent_key_path)
-                    ? " (a value of '$parent_key_path')"
-                    : "";
+    my $fh = $regfile->get_filehandle;
 
-    if (0) {
-        printf("NEW VALUE at 0x%x%s\n", $offset, $whereabouts);
-    }
-
-    my $fh = $regfile->{_filehandle};
-    croak "Missing filehandle" if !defined $fh;
-
-    # 0x00 dword = value length (as negative number)
+    # 0x00 dword = value length (negative = allocated)
     # 0x04 word  = 'vk' signature
     # 0x06 word  = value name length
-    # 0x08 dword = length of data (bit 31 set => data stored inline)
+    # 0x08 dword = value data length (bit 31 set => data stored inline)
     # 0x0c dword = offset to data/inline data
-    # 0x10 dword = type of data
-    # 0x14 word  = flag (bit 0 set = name present, unset = default)
+    # 0x10 dword = value type
+    # 0x14 word  = flags (bit 1 set => compressed name)
     # 0x16 word
-    # 0x18       = value name [for name length bytes]
+    # 0x18       = value name [for value name length bytes]
 
-    # Extracted offsets are always relative to first HBIN
+    # Extracted offsets are always relative to first hbin
 
     sysseek($fh, $offset, 0);
     my $bytes_read = sysread($fh, my $vk_header, VK_HEADER_LENGTH);
     if ($bytes_read != VK_HEADER_LENGTH) {
-        warnf("Could not read value at 0x%x%s", $offset, $whereabouts);
+        warnf('Could not read value at 0x%x', $offset);
         return;
     }
 
@@ -58,8 +47,8 @@ sub new {
         $data_length,
         $offset_to_data,
         $type,
-        $name_present_flag,
-        ) = unpack("Va2vVVVv", $vk_header);
+        $flags,
+        ) = unpack('Va2vVVVv', $vk_header);
 
     my $allocated = 0;
     if ($length > 0x7fffffff) {
@@ -68,20 +57,22 @@ sub new {
     }
     # allocated should be true
 
-    if ($sig ne "vk") {
-        warnf("Invalid signature for value at 0x%x%s", $offset, $whereabouts);
+    if ($sig ne 'vk') {
+        warnf('Invalid signature for value at 0x%x', $offset);
         return;
     }
 
-    my $name = "";
-    if ($name_present_flag & 1) {
-        $bytes_read = sysread($fh, $name, $name_length);
-        if ($bytes_read != $name_length) {
-            warnf("Could not read name for value at 0x%x%s",
-                $offset, $whereabouts);
-            return;
-        }
+    $bytes_read = sysread($fh, my $name, $name_length);
+    if ($bytes_read != $name_length) {
+        warnf('Could not read name for value at 0x%x', $offset);
+        return;
     }
+    if ($flags & 1) {
+        $name = decode($Parse::Win32Registry::Base::CODEPAGE, $name);
+    }
+    else {
+        $name = decode('UCS-2LE', $name);
+    };
 
     my $data;
 
@@ -93,8 +84,8 @@ sub new {
         # REG_SZ, REG_BINARY, REG_EXPAND_SZ, and REG_NONE inline
         $data_length &= 0x7fffffff;
         if ($data_length > 4) {
-            warnf("Invalid inline data length for value '%s' at 0x%x%s",
-                $name, $offset, $whereabouts);
+            warnf("Invalid inline data length for value '%s' at 0x%x",
+                $name, $offset);
             $data = undef;
         }
         else {
@@ -109,8 +100,8 @@ sub new {
         sysseek($fh, $offset_to_data + 4, 0);
         $bytes_read = sysread($fh, $data, $data_length);
         if ($bytes_read != $data_length) {
-            warnf("Could not read data at 0x%x for value '%s' at 0x%x%s",
-                $offset_to_data, $name, $offset, $whereabouts);
+            warnf("Could not read data at 0x%x for value '%s' at 0x%x",
+                $offset_to_data, $name, $offset);
             $data = undef;
         }
     }
@@ -122,11 +113,13 @@ sub new {
     $self->{_allocated} = $allocated;
     $self->{_tag} = $sig;
     $self->{_name} = $name;
+    $self->{_name_length} = $name_length;
     $self->{_type} = $type;
     $self->{_data} = $data;
     $self->{_data_length} = $data_length;
     $self->{_data_inline} = $data_inline;
     $self->{_offset_to_data} = $offset_to_data;
+    $self->{_flags} = $flags;
     bless $self, $class;
 
     return $self;
@@ -136,7 +129,6 @@ sub get_data {
     my $self = shift;
 
     my $type = $self->get_type;
-    croak "Missing type" if !defined $type;
 
     my $data = $self->{_data};
     return if !defined $data;
@@ -144,7 +136,7 @@ sub get_data {
     # apply decoding to appropriate data types
     if ($type == REG_DWORD) {
         if (length($data) == 4) {
-            $data = unpack("V", $data);
+            $data = unpack('V', $data);
         }
         else {
             # incorrect length for dword data
@@ -152,24 +144,18 @@ sub get_data {
         }
     }
     elsif ($type == REG_SZ || $type == REG_EXPAND_SZ) {
-        # handle unicode encoding
-        $data = decode("UCS-2LE", $data);
-
+        $data = decode('UCS-2LE', $data);
         # snip off any terminating null
-        if (substr($data, -1, 1) eq "\x00") {
-            chop $data;
-        }
+        chop $data if substr($data, -1, 1) eq "\0";
     }
     elsif ($type == REG_MULTI_SZ) {
-        my @multi_sz = ();
-        my $pos = 0;
-        do {
-            my ($str, $str_len) = unpack_unicode_string(substr($data, $pos));
-            push @multi_sz, $str;
-            $pos += $str_len;
-        } while ($pos < length($data));
-        # drop trailing empty string (caused by trailing null)
-        pop @multi_sz if @multi_sz > 1 && $multi_sz[-1] eq '';
+        $data = decode('UCS-2LE', $data);
+        # snip off any terminating nulls
+        chop $data if substr($data, -1, 1) eq "\0";
+        chop $data if substr($data, -1, 1) eq "\0";
+        my @multi_sz = split("\0", $data, -1);
+        # make sure there is at least one empty string
+        @multi_sz = ('') if @multi_sz == 0;
         return wantarray ? @multi_sz : join($", @multi_sz);
     }
 
@@ -181,12 +167,13 @@ sub as_regedit_export {
     my $version = shift || 5;
 
     my $name = $self->get_name;
-    my $export = $name eq "" ? "@=" : '"' . $name . '"=';
+    my $export = $name eq '' ? '@=' : '"' . $name . '"=';
 
     my $type = $self->get_type;
 
     if ($type == REG_SZ) {
         $export .= '"' . $self->get_data . '"';
+        $export .= "\n";
     }
     elsif ($type == REG_BINARY) {
         $export .= "hex:";
@@ -197,6 +184,7 @@ sub as_regedit_export {
         $export .= defined($data)
             ? sprintf("dword:%08x", $data)
             : "dword:";
+        $export .= "\n";
     }
     elsif ($type == REG_EXPAND_SZ || $type == REG_MULTI_SZ) {
         my $data = $version == 4
@@ -210,27 +198,24 @@ sub as_regedit_export {
         $export .= sprintf("hex(%x):", $type);
         $export .= format_octets($data, length($export));
     }
-    $export .= "\n";
     return $export;
 }
 
 sub parse_info {
     my $self = shift;
 
-    my $info = sprintf '0x%x,%d,%d vk "%s" type=%d (%s)',
+    my $info = sprintf '0x%x vk len=0x%x alloc=%d "%s" type=%d',
         $self->{_offset},
-        $self->{_allocated},
         $self->{_length},
+        $self->{_allocated},
         $self->{_name},
-        $self->{_type},
-        $self->get_type_as_string;
+        $self->{_type};
     if ($self->{_data_inline}) {
-        $info .= sprintf ' data=inline,%d bytes "%s"',
-            $self->{_data_length},
-            $self->get_data_as_string;
+        $info .= sprintf ' data=inline,len=0x%x',
+            $self->{_data_length};
     }
     else {
-        $info .= sprintf ' | data=0x%x,%d bytes',
+        $info .= sprintf ' data=0x%x,len=0x%x',
             $self->{_offset_to_data},
             $self->{_data_length};
     }
